@@ -33,6 +33,19 @@
 #include "guuid.h"
 #include "phodav-server.h"
 
+/**
+ * SECTION:phodav-server
+ * @title: PhodavServer
+ * @short_description: A WebDAV server
+ * @see_also: #SoupServer
+ * @stability: Stable
+ * @include: libphodav/phodav.h
+ *
+ * PhodavServer implements a simple WebDAV server.
+ */
+
+typedef struct _PathHandler PathHandler;
+
 struct _PhodavServer
 {
   GObject       parent;
@@ -41,8 +54,9 @@ struct _PhodavServer
   GThread      *thread;
   SoupServer   *server;
   GCancellable *cancellable;
-  gchar        *path;
-  gint          port;
+  gchar        *root;
+  PathHandler  *root_handler; /* weak ref */
+  guint         port;
   GHashTable   *paths;
 };
 
@@ -60,7 +74,7 @@ G_DEFINE_TYPE (PhodavServer, phodav_server, G_TYPE_OBJECT)
 enum {
   PROP_0,
   PROP_PORT,
-  PROP_PATH,
+  PROP_ROOT,
   PROP_SERVER,
 };
 
@@ -229,11 +243,11 @@ dav_lock_free (DAVLock *lock)
   g_slice_free (DAVLock, lock);
 }
 
-typedef struct _PathHandler
+struct _PathHandler
 {
   PhodavServer *self;
   GFile       *file;
-} PathHandler;
+};
 
 static PathHandler*
 path_handler_new (PhodavServer *self, GFile *file)
@@ -291,22 +305,36 @@ phodav_server_init (PhodavServer *self)
 }
 
 static void
-phodav_server_constructed (GObject *gobject)
+update_root_handler (PhodavServer *self)
 {
-  PhodavServer *self = PHODAV_SERVER (gobject);
-  PathHandler *handler = path_handler_new (self, g_file_new_for_path (self->path));
+  PathHandler *handler;
 
-  self->server = soup_server_new (SOUP_SERVER_PORT, self->port,
-                                  SOUP_SERVER_SERVER_HEADER, "PhodavServer ",
-                                  SOUP_SERVER_ASYNC_CONTEXT, self->context,
-                                  NULL);
+  if (!self->root || !self->server)
+    return;
+
+  handler = path_handler_new (self, g_file_new_for_path (self->root));
 
   soup_server_add_handler (self->server, NULL,
                            server_callback,
                            handler,
                            (GDestroyNotify) path_handler_free);
 
-  g_signal_connect (self->server, "request-started", G_CALLBACK (request_started), handler);
+  self->root_handler = handler;
+}
+
+static void
+phodav_server_constructed (GObject *gobject)
+{
+  PhodavServer *self = PHODAV_SERVER (gobject);
+
+  self->server = soup_server_new (SOUP_SERVER_PORT, self->port,
+                                  SOUP_SERVER_SERVER_HEADER, "PhodavServer ",
+                                  SOUP_SERVER_ASYNC_CONTEXT, self->context,
+                                  NULL);
+
+  update_root_handler (self);
+
+  g_signal_connect (self->server, "request-started", G_CALLBACK (request_started), self);
 
   /* Chain up to the parent class */
   if (G_OBJECT_CLASS (phodav_server_parent_class)->constructed)
@@ -318,7 +346,7 @@ phodav_server_dispose (GObject *gobject)
 {
   PhodavServer *self = PHODAV_SERVER (gobject);
 
-  g_clear_pointer (&self->path, g_free);
+  g_clear_pointer (&self->root, g_free);
   g_clear_pointer (&self->context, g_main_context_unref);
   g_clear_pointer (&self->thread, g_thread_unref);
   g_clear_pointer (&self->paths, g_hash_table_unref);
@@ -339,11 +367,11 @@ phodav_server_get_property (GObject    *gobject,
   switch (prop_id)
     {
     case PROP_PORT:
-      g_value_set_int (value, phodav_server_get_port (self));
+      g_value_set_uint (value, phodav_server_get_port (self));
       break;
 
-    case PROP_PATH:
-      g_value_set_string (value, self->path);
+    case PROP_ROOT:
+      g_value_set_string (value, self->root);
       break;
 
     case PROP_SERVER:
@@ -367,12 +395,13 @@ phodav_server_set_property (GObject      *gobject,
   switch (prop_id)
     {
     case PROP_PORT:
-      self->port = g_value_get_int (value);
+      self->port = g_value_get_uint (value);
       break;
 
-    case PROP_PATH:
-      g_free (self->path);
-      self->path = g_value_dup_string (value);
+    case PROP_ROOT:
+      g_free (self->root);
+      self->root = g_value_dup_string (value);
+      update_root_handler (self);
       break;
 
     default:
@@ -393,20 +422,20 @@ phodav_server_class_init (PhodavServerClass *klass)
 
   g_object_class_install_property
     (gobject_class, PROP_PORT,
-    g_param_spec_int ("port",
-                      "Port",
-                      "Port",
-                      0, G_MAXINT16, 0,
-                      G_PARAM_CONSTRUCT_ONLY |
-                      G_PARAM_READWRITE |
-                      G_PARAM_STATIC_STRINGS));
+    g_param_spec_uint ("port",
+                       "Port",
+                       "Port",
+                       0, G_MAXUINT16, 0,
+                       G_PARAM_CONSTRUCT_ONLY |
+                       G_PARAM_READWRITE |
+                       G_PARAM_STATIC_STRINGS));
 
   g_object_class_install_property
-    (gobject_class, PROP_PATH,
-    g_param_spec_string ("path",
-                         "Path",
-                         "Path",
-                         ".",
+    (gobject_class, PROP_ROOT,
+    g_param_spec_string ("root",
+                         "Root path",
+                         "Root path",
+                         NULL,
                          G_PARAM_CONSTRUCT |
                          G_PARAM_READWRITE |
                          G_PARAM_STATIC_STRINGS));
@@ -414,8 +443,8 @@ phodav_server_class_init (PhodavServerClass *klass)
   g_object_class_install_property
     (gobject_class, PROP_SERVER,
      g_param_spec_object ("server",
-                          "Server",
-                          "Server",
+                          "Soup Server",
+                          "Soup Server",
                           SOUP_TYPE_SERVER,
                           G_PARAM_READABLE |
                           G_PARAM_STATIC_STRINGS));
@@ -2965,13 +2994,13 @@ static void
 got_headers (SoupMessage *msg,
              gpointer     user_data)
 {
-  PathHandler *handler = user_data;
+  PhodavServer *self = user_data;
   SoupURI *uri = soup_message_get_uri (msg);
   const gchar *path = uri->path;
   GError *err = NULL;
 
   if (msg->method == SOUP_METHOD_PUT)
-    method_put (handler, path, msg, &err);
+    method_put (self->root_handler, path, msg, &err);
 
   if (err)
     {
@@ -2986,9 +3015,9 @@ request_started (SoupServer        *server,
                  SoupClientContext *client,
                  gpointer           user_data)
 {
-  PathHandler *handler = user_data;
+  PhodavServer *self = user_data;
 
-  g_signal_connect (message, "got-headers", G_CALLBACK (got_headers), handler);
+  g_signal_connect (message, "got-headers", G_CALLBACK (got_headers), self);
 }
 
 static void
@@ -3065,14 +3094,31 @@ server_callback (SoupServer *server, SoupMessage *msg,
     }
 }
 
-gint
+/**
+ * phodav_server_get_port:
+ * @server: a %PhodavServer
+ *
+ * Gets the TCP port that server is listening on. This is most useful
+ * when you did not request a specific port, with value 0.
+*
+ * Returns: the port @server is listening on.
+ **/
+guint
 phodav_server_get_port (PhodavServer *self)
 {
-  g_return_val_if_fail (PHODAV_IS_SERVER (self), -1);
+  g_return_val_if_fail (PHODAV_IS_SERVER (self), 0);
 
   return soup_server_get_port (self->server);
 }
 
+/**
+ * phodav_server_get_soup_server:
+ * @server: a %PhodavServer
+ *
+ * Returns the underlying %SoupServer, if any.
+ *
+ * Returns: the associated %SoupServer or %NULL
+ **/
 SoupServer *
 phodav_server_get_soup_server (PhodavServer *self)
 {
@@ -3086,7 +3132,7 @@ thread_func (gpointer data)
 {
   PhodavServer *self = data;
 
-  g_debug ("Starting on port %d, serving %s", phodav_server_get_port (self), self->path);
+  g_debug ("Starting on port %u, serving %s", phodav_server_get_port (self), self->root);
 
   soup_server_run_async (self->server);
 
@@ -3095,6 +3141,12 @@ thread_func (gpointer data)
   return NULL;
 }
 
+/**
+ * phodav_server_run:
+ * @server: a %PhodavServer
+ *
+ * Run the server in a separate thread.
+ **/
 void
 phodav_server_run (PhodavServer *self)
 {
@@ -3107,6 +3159,12 @@ phodav_server_run (PhodavServer *self)
   self->thread = g_thread_new ("phodav-server", thread_func, self);
 }
 
+/**
+ * phodav_server_quit:
+ * @server: a %PhodavServer
+ *
+ * Stops the server from running.
+ **/
 void
 phodav_server_quit (PhodavServer *self)
 {
@@ -3122,12 +3180,20 @@ phodav_server_quit (PhodavServer *self)
   g_object_unref (self);
 }
 
+/**
+ * phodav_server_new:
+ * @port: Port to listen on.
+ * @root: (allow-none): Root path.
+ *
+ * Creates a new #PhodavServer.
+ *
+ * Returns: a new #PhodavServer
+ **/
 PhodavServer *
-phodav_server_new (gint port, const gchar *path)
+phodav_server_new (guint port, const gchar *root)
 {
-  if (path == NULL)
-    path = ".";
-
   return g_object_new (PHODAV_TYPE_SERVER,
-                       "port", port, "path", path, NULL);
+                       "port", port,
+                       "root", root,
+                       NULL);
 }
